#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>


static volatile int mode = 0;
static volatile int pulse = 0;

int alarm;

ISR(PCINT0_vect) {
	if (mode == 1) {
		/* Disable counter */
		TCCR1B = 0;
		pulse = TCNT1;
		TCNT1 = 0;
		mode = 0;
	}
	if (mode == 0) {
		TCCR1B |= (1 << CS11);
		mode = 1;
	}
}


void IO_init(void) {
	/* Enable Interrupts */
	sei();
	PCICR |= (1 << PCIE0) | (1 << PCIE3);
	PCMSK0 |= (1 << PCINT0);
	PCMSK3 |= (1 << PCINT30);

	/* Pin Config */
	DDRD |= (1 << PD0) | (1 << PD7);
	
	/* Setez pinul PB0 ca intrare. */
	DDRD &= ~(1 << PD6);

	/* Activez rezistenta de pull-up pentru pinul PB0. */
	PORTD |= (1 << PD6);
	
	/* Setez pinul PB4 ca pin de iesire (buzzer) */
	DDRD |= (1 << PD1);
	/* Opresc buzzer-ul */
	PORTD &= ~(1 << PD1);
}


void led_blink(int distance) {
	int steps = 500 / (distance * 5);
	DDRD |= (1 << PD1);
	int i;
	for (i = 0; i < steps; i++) {
		PORTD ^= (1 << PD1);
		PORTD ^= (1 << PD7);
		_delay_ms(distance);
	}
	PORTD &= ~(1 << PD1);
	PORTD &= ~(1 << PD7);
}


void LED_init() {

	/* Setez pinii PC0-PC5 ca pin de iesire (semafoare) */
	DDRC |= (7 << PC0);
	PORTC &= ~(7 << PC0);

}


void compute_distance() {

		// Write to TRIG pin
		PORTD |= (1 << PD0);
		_delay_us(15);
		PORTD &= ~(1 << PD0);
		
		int distance, aux, digits;

			distance = pulse / (53 * 2);
			if (distance == 0)
				return;
			aux = distance;
			digits = 0;
			while (aux) {
				aux /= 10;
				digits++;
			}
			if (distance <= 10 && distance > 0 && alarm == 1) {
				led_blink(distance);
			}
			else {

				DDRD &= ~(1 << PD1);
				PORTD &= ~(1 << PD7);
				_delay_ms(100);
			}
}



int state(int number)
{
	int i, j;
	
	/* modul normal de rulare al intersectiei are 9 stari */
	switch (number)
	{
		/* starea 0 este un rosu general de 2 secunde */
		case 0:
			PORTB = 0b00001001;
			PORTC = 0b00001001;
			
			alarm = 1;
			
			for (i = 0; i < 200; i++)
			{
				/* se verifica la fiecare milisecunda daca s-a schimbat starea switch-ului */
				//_delay_ms(100);
				compute_distance();
			}
			break;
		
		/* starea 1 -> S1 verde, S2 rosu - dureaza 10 secunde */
		case 1:
			PORTB = 0b00100100;
			PORTC = 0b00001001;
			
			alarm = 0;
			
			for (i = 0; i < 1000; i++)
			{
				//_delay_ms(100);
				compute_distance();
			}
			break;
		
		/* starea 2 -> S1 verde clipit, S2 rosu - dureaza 4 secunde */
		case 2:
			PORTB = 0b00100100;
			PORTC = 0b00001001;
			
			alarm = 0;
			
			for (i = 0; i < 8; i ++)
			{
				PORTB ^= (1 << PB2);
				PORTB ^= (1 << PB5);
				
				/* odata la 500 milisecunde se va schimba starea LED-urilor verzi de la pietoni ce sunt in starea de clipire */
				for (j = 0; j < 50; j ++)
				{
					//_delay_ms(100);
					compute_distance();
				}
			}
			break;
		

		/* starea 3 -> S1 galben, S2 rosu - dureaza 3 secunde */
		case 3:
			PORTB = 0b00010010;
			PORTC = 0b00001001;
			
			alarm = 0;
			
			for (i = 0; i < 300; i ++)
			{
				//_delay_ms(100);
				compute_distance();
			}
			break;
		
		/* starea 4 este un rosu general de 2 secunde */
		case 4:
			PORTB = 0b00001001;
			PORTC = 0b00001001;
			
			alarm = 1;
			
			for (i = 0; i < 200; i ++)
			{
				//_delay_ms(100);
				compute_distance();
			}
			break;
		
		/* starea 5 -> S1 rosu, S2 verde - dureaza 12 secunde */
		case 5:
			PORTB = 0b00001001;
			PORTC = 0b00100100;
			
			alarm = 1;
			
			for (i = 0; i < 1000; i ++)
			{
				//_delay_ms(100);
				compute_distance();
			}
			break;
		
		/* starea 6 -> S1 rosu, S2 verde clipit - dureaza 4 secunde */
		case 6:
			PORTB = 0b00001001;
			PORTC = 0b00100100;
			
			alarm = 1;
			
			for (i = 0; i < 8; i ++)
			{
				PORTC ^= (1 << PC2);
				PORTC ^= (1 << PC5);
				
				for (j = 0; j < 50; j ++)
				{
					//_delay_ms(100);
					compute_distance();
				}
			}
			break;
		
	
		/* starea 7 -> S1 rosu, S2 galben - dureaza 3 secunde */
		case 7:
			PORTB = 0b00001001;
			PORTC = 0b00010010;
			
			alarm = 1;
			
			for (i = 0; i < 300; i ++)
			{
				//_delay_ms(100);
				compute_distance();
			}
			break;
	}
	
	return 1;
}

int main(void) {
	int i;

	IO_init();
	
	/* activam aproape toti pinii de pe PORTA si PORTB ca pini de iesire, deoarece acolo se afla conectate LED-urile */
	DDRB |= 255;
	DDRB &= ~(1 << PA6);
	DDRB &= ~(1 << PA7);
	
	DDRC |= 255;
	
	alarm = 0;


	while(1) {

		for (i = 0; i < 8; i++) {
			state(i);
		}
		//compute_distance();
	}
}